@@ -2,7 +2,8 @@
 #  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Copyright (c) 2008-2022
+#  National Technology and Engineering Solutions of Sandia, LLC
 #  Under the terms of Contract DE-NA0003525 with National Technology and
 #  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain
 #  rights in this software.
@@ -63,8 +64,13 @@ class TestTeeStream(unittest.TestCase):
             start_time = time.time()
             while 'cow' not in a.getvalue() and time.time() - start_time < 1:
                 time.sleep(tee._poll_interval)
-        self.assertEqual(a.getvalue(), "Hello\ninterrupting\ncowWorld")
-        self.assertEqual(b.getvalue(), "Hello\ninterrupting\ncowWorld")
+        acceptable_results = {
+            "Hello\ninterrupting\ncowWorld", # expected
+            "interrupting\ncowHello\nWorld", # Windows occasionally puts
+                                             # all error before stdout
+        }
+        self.assertIn(a.getvalue(), acceptable_results)
+        self.assertEqual(b.getvalue(), a.getvalue())
 
     def test_merged_out_and_err_without_peek(self):
         a = StringIO()
@@ -238,7 +244,18 @@ class TestFileDescriptor(unittest.TestCase):
             self.assertEqual(FILE.read(),
                              "to_stdout_1\nto_stdout_2\nto_fd1_2\n")
 
+    # Pytest's default capture method causes failures for the following
+    # two tests. This re-implementation of the capfd fixture allows
+    # the capture to be disabled for those two test specifically.
+    @unittest.pytest.fixture(autouse=True)
+    def capfd(self, capfd):
+        """
+        Reimplementation needed for use in unittest.TestCase subclasses
+        """
+        self.capfd = capfd
+
     def test_redirect_synchronize_stdout_not_fd1(self):
+        self.capfd.disabled()
         r,w = os.pipe()
         os.dup2(w, 1)
         rd = tee.redirect_fd(synchronize=True)
@@ -250,6 +267,7 @@ class TestFileDescriptor(unittest.TestCase):
             self.assertEqual(FILE.read(), "to_fd1_2\n")
 
     def test_redirect_no_synchronize_stdout_not_fd1(self):
+        self.capfd.disabled()
         r,w = os.pipe()
         os.dup2(w, 1)
         rd = tee.redirect_fd(synchronize=False)
