@@ -1,9 +1,10 @@
 #  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
-#  Under the terms of Contract DE-NA0003525 with National Technology and 
-#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  Copyright (c) 2008-2022
+#  National Technology and Engineering Solutions of Sandia, LLC
+#  Under the terms of Contract DE-NA0003525 with National Technology and
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain
 #  rights in this software.
 #  This software is distributed under the 3-clause BSD License.
 #  ___________________________________________________________________________
@@ -15,7 +16,8 @@ from pyomo.common.dependencies import numpy as np, numpy_available
 from pyomo.common.log import LoggingIntercept
 from pyomo.common.errors import InfeasibleConstraintException
 from pyomo.core.expr.numeric_expr import (ProductExpression,
-                                          UnaryFunctionExpression)
+                                          UnaryFunctionExpression,
+                                          LinearExpression)
 import math
 from io import StringIO
 
@@ -24,9 +26,21 @@ class DummyExpr(ProductExpression):
     pass
 
 
-class TestFBBT(unittest.TestCase):
-    @unittest.skipIf(not numpy_available, 'Numpy is not available.')
+class FbbtTestBase(object):
+    """
+    These tests are set up weird, but it is for a good reason.
+    The FBBT code is duplicated in pyomo.contrib.appsi for
+    improved performance. We want to keep this version because
+    it does not require building an extension. However, when we
+    fix a bug in one module, we want to ensure we fix that bug
+    in the other module. Therefore, we use this base class
+    for testing both modules. The only difference in the
+    derived classes is self.tightener attribute.
+    """
+
     def test_add(self):
+        if not numpy_available:
+            raise unittest.SkipTest('Numpy is not available')
         x_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         c_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         for xl, xu in x_bounds:
@@ -37,9 +51,7 @@ class TestFBBT(unittest.TestCase):
                 m.p = pyo.Param(mutable=True)
                 m.p.value = 1
                 m.c = pyo.Constraint(expr=pyo.inequality(body=m.x+m.y+(m.p+1), lower=cl, upper=cu))
-                new_bounds = fbbt(m)
-                self.assertEqual(new_bounds[m.x], (pyo.value(m.x.lb), pyo.value(m.x.ub)))
-                self.assertEqual(new_bounds[m.y], (pyo.value(m.y.lb), pyo.value(m.y.ub)))
+                self.tightener(m)
                 x = np.linspace(pyo.value(m.x.lb), pyo.value(m.x.ub), 100)
                 z = np.linspace(pyo.value(m.c.lower), pyo.value(m.c.upper), 100)
                 if m.y.lb is None:
@@ -55,8 +67,9 @@ class TestFBBT(unittest.TestCase):
                     self.assertTrue(np.all(yl <= _y))
                     self.assertTrue(np.all(yu >= _y))
 
-    @unittest.skipIf(not numpy_available, 'Numpy is not available.')
     def test_sub1(self):
+        if not numpy_available:
+            raise unittest.SkipTest('Numpy is not available')
         x_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         c_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         for xl, xu in x_bounds:
@@ -65,7 +78,7 @@ class TestFBBT(unittest.TestCase):
                 m.x = pyo.Var(bounds=(xl, xu))
                 m.y = pyo.Var()
                 m.c = pyo.Constraint(expr=pyo.inequality(body=m.x-m.y, lower=cl, upper=cu))
-                fbbt(m)
+                self.tightener(m)
                 x = np.linspace(pyo.value(m.x.lb), pyo.value(m.x.ub), 100)
                 z = np.linspace(pyo.value(m.c.lower), pyo.value(m.c.upper), 100)
                 if m.y.lb is None:
@@ -81,8 +94,9 @@ class TestFBBT(unittest.TestCase):
                     self.assertTrue(np.all(yl <= _y))
                     self.assertTrue(np.all(yu >= _y))
 
-    @unittest.skipIf(not numpy_available, 'Numpy is not available.')
     def test_sub2(self):
+        if not numpy_available:
+            raise unittest.SkipTest('Numpy is not available')
         x_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         c_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         for xl, xu in x_bounds:
@@ -91,7 +105,7 @@ class TestFBBT(unittest.TestCase):
                 m.x = pyo.Var(bounds=(xl, xu))
                 m.y = pyo.Var()
                 m.c = pyo.Constraint(expr=pyo.inequality(body=m.y-m.x, lower=cl, upper=cu))
-                fbbt(m)
+                self.tightener(m)
                 x = np.linspace(pyo.value(m.x.lb), pyo.value(m.x.ub), 100)
                 z = np.linspace(pyo.value(m.c.lower), pyo.value(m.c.upper), 100)
                 if m.y.lb is None:
@@ -107,8 +121,9 @@ class TestFBBT(unittest.TestCase):
                     self.assertTrue(np.all(yl <= _y))
                     self.assertTrue(np.all(yu >= _y))
 
-    @unittest.skipIf(not numpy_available, 'Numpy is not available.')
     def test_mul(self):
+        if not numpy_available:
+            raise unittest.SkipTest('Numpy is not available')
         x_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         c_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         for xl, xu in x_bounds:
@@ -117,7 +132,7 @@ class TestFBBT(unittest.TestCase):
                 m.x = pyo.Var(bounds=(xl, xu))
                 m.y = pyo.Var()
                 m.c = pyo.Constraint(expr=pyo.inequality(body=m.x*m.y, lower=cl, upper=cu))
-                fbbt(m)
+                self.tightener(m)
                 x = np.linspace(pyo.value(m.x.lb) + 1e-6, pyo.value(m.x.ub), 100, endpoint=False)
                 z = np.linspace(pyo.value(m.c.lower), pyo.value(m.c.upper), 100)
                 if m.y.lb is None:
@@ -133,8 +148,9 @@ class TestFBBT(unittest.TestCase):
                     self.assertTrue(np.all(yl <= _y))
                     self.assertTrue(np.all(yu >= _y))
 
-    @unittest.skipIf(not numpy_available, 'Numpy is not available.')
     def test_div1(self):
+        if not numpy_available:
+            raise unittest.SkipTest('Numpy is not available')
         x_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         c_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         for xl, xu in x_bounds:
@@ -143,7 +159,7 @@ class TestFBBT(unittest.TestCase):
                 m.x = pyo.Var(bounds=(xl, xu))
                 m.y = pyo.Var()
                 m.c = pyo.Constraint(expr=pyo.inequality(body=m.x/m.y, lower=cl, upper=cu))
-                fbbt(m)
+                self.tightener(m)
                 x = np.linspace(pyo.value(m.x.lb), pyo.value(m.x.ub), 100)
                 z = np.linspace(pyo.value(m.c.lower) + 1e-6, pyo.value(m.c.upper), 100, endpoint=False)
                 if m.y.lb is None:
@@ -159,8 +175,9 @@ class TestFBBT(unittest.TestCase):
                     self.assertTrue(np.all(yl <= _y))
                     self.assertTrue(np.all(yu >= _y))
 
-    @unittest.skipIf(not numpy_available, 'Numpy is not available.')
     def test_div2(self):
+        if not numpy_available:
+            raise unittest.SkipTest('Numpy is not available')
         x_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         c_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         for xl, xu in x_bounds:
@@ -169,7 +186,7 @@ class TestFBBT(unittest.TestCase):
                 m.x = pyo.Var(bounds=(xl, xu))
                 m.y = pyo.Var()
                 m.c = pyo.Constraint(expr=pyo.inequality(body=m.y/m.x, lower=cl, upper=cu))
-                fbbt(m)
+                self.tightener(m)
                 x = np.linspace(pyo.value(m.x.lb), pyo.value(m.x.ub), 100)
                 z = np.linspace(pyo.value(m.c.lower), pyo.value(m.c.upper), 100)
                 if m.y.lb is None:
@@ -185,8 +202,9 @@ class TestFBBT(unittest.TestCase):
                     self.assertTrue(np.all(yl <= _y))
                     self.assertTrue(np.all(yu >= _y))
 
-    @unittest.skipIf(not numpy_available, 'Numpy is not available.')
     def test_pow1(self):
+        if not numpy_available:
+            raise unittest.SkipTest('Numpy is not available')
         x_bounds = [(0, 2.8), (0.5, 2.8), (1, 2.8), (0.5, 1)]
         c_bounds = [(-2.5, 2.8), (0.5, 2.8), (-2.5, 0), (0, 2.8), (1, 2.8), (0.5, 1)]
         for xl, xu in x_bounds:
@@ -197,9 +215,9 @@ class TestFBBT(unittest.TestCase):
                 m.c = pyo.Constraint(expr=pyo.inequality(body=m.x**m.y, lower=cl, upper=cu))
                 if xl > 0 and cu <= 0:
                     with self.assertRaises(InfeasibleConstraintException):
-                        fbbt(m)
+                        self.tightener(m)
                 else:
-                    fbbt(m)
+                    self.tightener(m)
                     x = np.linspace(pyo.value(m.x.lb) + 1e-6, pyo.value(m.x.ub), 100, endpoint=False)
                     z = np.linspace(pyo.value(m.c.lower) + 1e-6, pyo.value(m.c.upper), 100, endpoint=False)
                     if m.y.lb is None:
@@ -215,8 +233,9 @@ class TestFBBT(unittest.TestCase):
                         self.assertTrue(np.all(yl <= _y))
                         self.assertTrue(np.all(yu >= _y))
 
-    @unittest.skipIf(not numpy_available, 'Numpy is not available.')
     def test_pow2(self):
+        if not numpy_available:
+            raise unittest.SkipTest('Numpy is not available')
         x_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         c_bounds = [(-2.5, 2.8), (0.5, 2.8), (0, 2.8), (1, 2.8), (0.5, 1)]
         for xl, xu in x_bounds:
@@ -225,7 +244,7 @@ class TestFBBT(unittest.TestCase):
                 m.x = pyo.Var(bounds=(xl, xu))
                 m.y = pyo.Var()
                 m.c = pyo.Constraint(expr=pyo.inequality(body=m.y**m.x, lower=cl, upper=cu))
-                fbbt(m)
+                self.tightener(m)
                 x = np.linspace(pyo.value(m.x.lb) + 1e-6, pyo.value(m.x.ub), 100, endpoint=False)
                 z = np.linspace(pyo.value(m.c.lower) + 1e-6, pyo.value(m.c.upper), 100, endpoint=False)
                 if m.y.lb is None:
@@ -246,7 +265,7 @@ class TestFBBT(unittest.TestCase):
         m.y = pyo.Var()
         m.c = pyo.Constraint(expr=m.x**2 == m.y)
 
-        fbbt(m)
+        self.tightener(m)
         self.assertEqual(m.x.lb, None)
         self.assertEqual(m.x.ub, None)
         self.assertEqual(m.y.lb, 0)
@@ -256,28 +275,28 @@ class TestFBBT(unittest.TestCase):
         m.x.setub(None)
         m.y.setlb(1)
         m.y.setub(4)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, -2)
         self.assertAlmostEqual(m.x.ub, 2)
 
         m.x.setlb(0)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, 1)
         self.assertAlmostEqual(m.x.ub, 2)
 
         m.x.setlb(-0.5)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, 1)
         self.assertAlmostEqual(m.x.ub, 2)
 
         m.x.setlb(-1)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, -1)
         self.assertAlmostEqual(m.x.ub, 2)
 
         m.x.setlb(None)
         m.x.setub(0)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, -2)
         self.assertAlmostEqual(m.x.ub, -1)
 
@@ -286,10 +305,10 @@ class TestFBBT(unittest.TestCase):
         m.y.setlb(-5)
         m.y.setub(-1)
         with self.assertRaises(InfeasibleConstraintException):
-            fbbt(m)
+            self.tightener(m)
 
         m.y.setub(0)
-        fbbt(m)
+        self.tightener(m)
         self.assertEqual(m.x.lb, 0)
         self.assertEqual(m.x.ub, 0)
 
@@ -299,7 +318,7 @@ class TestFBBT(unittest.TestCase):
         m.y = pyo.Var(bounds=(0.5, 1))
         m.c = pyo.Constraint(expr=2**m.x == m.y)
 
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, -1)
         self.assertAlmostEqual(m.x.ub, 0)
 
@@ -309,7 +328,7 @@ class TestFBBT(unittest.TestCase):
         m.y = pyo.Var()
         m.c = pyo.Constraint(expr=m.x**(-2) == m.y)
 
-        fbbt(m)
+        self.tightener(m)
         self.assertEqual(m.x.lb, None)
         self.assertEqual(m.x.ub, None)
         self.assertEqual(m.y.lb, 0)
@@ -318,30 +337,30 @@ class TestFBBT(unittest.TestCase):
         m.y.setlb(-5)
         m.y.setub(-1)
         with self.assertRaises(InfeasibleConstraintException):
-            fbbt(m)
+            self.tightener(m)
 
         m.x.setlb(None)
         m.x.setub(None)
         m.y.setub(0)
         with self.assertRaises(InfeasibleConstraintException):
-            fbbt(m)
+            self.tightener(m)
 
         m.x.setlb(None)
         m.x.setub(None)
         m.y.setub(1)
         m.y.setlb(0.25)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, -2)
         self.assertAlmostEqual(m.x.ub, 2)
 
         m.x.setlb(0)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, 1)
         self.assertAlmostEqual(m.x.ub, 2)
 
         m.x.setlb(None)
         m.x.setub(0)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, -2)
         self.assertAlmostEqual(m.x.ub, -1)
 
@@ -351,7 +370,7 @@ class TestFBBT(unittest.TestCase):
         m.y = pyo.Var()
         m.c = pyo.Constraint(expr=m.x**3 == m.y)
 
-        fbbt(m)
+        self.tightener(m)
         self.assertEqual(m.x.lb, None)
         self.assertEqual(m.x.ub, None)
         self.assertEqual(m.y.lb, None)
@@ -361,7 +380,7 @@ class TestFBBT(unittest.TestCase):
         m.x.setub(None)
         m.y.setlb(1)
         m.y.setub(8)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, 1)
         self.assertAlmostEqual(m.x.ub, 2)
 
@@ -369,7 +388,7 @@ class TestFBBT(unittest.TestCase):
         m.x.setub(None)
         m.y.setlb(-8)
         m.y.setub(8)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, -2)
         self.assertAlmostEqual(m.x.ub, 2)
 
@@ -377,7 +396,7 @@ class TestFBBT(unittest.TestCase):
         m.x.setub(None)
         m.y.setlb(-5)
         m.y.setub(8)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, -5.0**(1.0/3.0))
         self.assertAlmostEqual(m.x.ub, 2)
 
@@ -385,7 +404,7 @@ class TestFBBT(unittest.TestCase):
         m.x.setub(None)
         m.y.setlb(-8)
         m.y.setub(-1)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, -2)
         self.assertAlmostEqual(m.x.ub, -1)
 
@@ -395,7 +414,7 @@ class TestFBBT(unittest.TestCase):
         m.y = pyo.Var()
         m.c = pyo.Constraint(expr=m.x**(-3) == m.y)
 
-        fbbt(m)
+        self.tightener(m)
         self.assertEqual(m.x.lb, None)
         self.assertEqual(m.x.ub, None)
         self.assertEqual(m.y.lb, None)
@@ -403,14 +422,14 @@ class TestFBBT(unittest.TestCase):
 
         m.y.setlb(-1)
         m.y.setub(-0.125)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, -2)
         self.assertAlmostEqual(m.x.ub, -1)
 
         m.x.setlb(None)
         m.x.setub(None)
         m.y.setub(0)
-        fbbt(m)
+        self.tightener(m)
         self.assertEqual(m.x.lb, None)
         self.assertAlmostEqual(m.x.ub, -1)
 
@@ -418,18 +437,19 @@ class TestFBBT(unittest.TestCase):
         m.x.setub(None)
         m.y.setlb(-1)
         m.y.setub(1)
-        fbbt(m)
+        self.tightener(m)
         self.assertEqual(m.x.lb, None)
         self.assertEqual(m.x.ub, None)
 
         m.y.setlb(0.125)
         m.y.setub(1)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, 1)
         self.assertAlmostEqual(m.x.ub, 2)
 
-    @unittest.skipIf(not numpy_available, 'Numpy is not available.')
     def test_pow4(self):
+        if not numpy_available:
+            raise unittest.SkipTest('Numpy is not available')
         y_bounds = [(0.5, 2.8), (0, 2.8), (1, 2.8), (0.5, 1), (0, 0.5)]
         exp_vals = [-3, -2.5, -2, -1.5, -1, -0.5, 0.5, 1, 1.5, 2, 2.5, 3]
         for yl, yu in y_bounds:
@@ -438,7 +458,7 @@ class TestFBBT(unittest.TestCase):
                 m.x = pyo.Var()
                 m.y = pyo.Var(bounds=(yl, yu))
                 m.c = pyo.Constraint(expr=m.x**_exp_val == m.y)
-                fbbt(m)
+                self.tightener(m)
                 y = np.linspace(pyo.value(m.y.lb) + 1e-6, pyo.value(m.y.ub), 100, endpoint=True)
                 if m.x.lb is None:
                     xl = -np.inf
@@ -458,7 +478,7 @@ class TestFBBT(unittest.TestCase):
         m.y = pyo.Var()
         m.c = pyo.Constraint(expr=pyo.sqrt(m.x) == m.y)
 
-        fbbt(m)
+        self.tightener(m)
         self.assertEqual(m.x.lb, 0)
         self.assertEqual(m.x.ub, None)
         self.assertEqual(m.y.lb, 0)
@@ -469,13 +489,13 @@ class TestFBBT(unittest.TestCase):
         m.y.setlb(-5)
         m.y.setub(-1)
         with self.assertRaises(InfeasibleConstraintException):
-            fbbt(m)
+            self.tightener(m)
 
         m.x.setlb(None)
         m.x.setub(None)
         m.y.setub(0)
         m.y.setlb(None)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, 0)
         self.assertAlmostEqual(m.x.ub, 0)
 
@@ -483,7 +503,7 @@ class TestFBBT(unittest.TestCase):
         m.x.setub(None)
         m.y.setub(2)
         m.y.setlb(1)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.x.lb, 1)
         self.assertAlmostEqual(m.x.ub, 4)
 
@@ -491,18 +511,19 @@ class TestFBBT(unittest.TestCase):
         m.x.setub(0)
         m.y.setlb(None)
         m.y.setub(None)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(m.y.lb, 0)
         self.assertAlmostEqual(m.y.ub, 0)
 
-    @unittest.skipIf(not numpy_available, 'Numpy is not available.')
     def test_exp(self):
+        if not numpy_available:
+            raise unittest.SkipTest('Numpy is not available')
         c_bounds = [(-2.5, 2.8), (0.5, 2.8), (0, 2.8), (1, 2.8), (0.5, 1)]
         for cl, cu in c_bounds:
             m = pyo.Block(concrete=True)
             m.x = pyo.Var()
             m.c = pyo.Constraint(expr=pyo.inequality(body=pyo.exp(m.x), lower=cl, upper=cu))
-            fbbt(m)
+            self.tightener(m)
             if pyo.value(m.c.lower) <= 0:
                 _cl = 1e-6
             else:
@@ -520,14 +541,95 @@ class TestFBBT(unittest.TestCase):
             self.assertTrue(np.all(xl <= x))
             self.assertTrue(np.all(xu >= x))
 
-    @unittest.skipIf(not numpy_available, 'Numpy is not available.')
+    def test_abs(self):
+        m = pyo.ConcreteModel()
+        m.x = pyo.Var()
+        m.y = pyo.Var()
+        m.c = pyo.Constraint(expr=abs(m.x) == m.y)
+
+        self.tightener(m)
+        self.assertEqual(m.x.lb, None)
+        self.assertEqual(m.x.ub, None)
+        self.assertAlmostEqual(m.y.lb, 0)
+        self.assertEqual(m.y.ub, None)
+
+        m.x.setlb(None)
+        m.x.setub(None)
+        m.y.setlb(2)
+        m.y.setub(4)
+        self.tightener(m)
+        self.assertAlmostEqual(m.x.lb, -4)
+        self.assertAlmostEqual(m.x.ub, 4)
+        self.assertAlmostEqual(m.y.lb, 2)
+        self.assertAlmostEqual(m.y.ub, 4)
+
+        m.x.setlb(None)
+        m.x.setub(None)
+        m.y.setlb(-5)
+        m.y.setub(4)
+        self.tightener(m)
+        self.assertAlmostEqual(m.x.lb, -4)
+        self.assertAlmostEqual(m.x.ub, 4)
+        self.assertAlmostEqual(m.y.lb, 0)
+        self.assertAlmostEqual(m.y.ub, 4)
+
+        m.x.setlb(None)
+        m.x.setub(None)
+        m.y.setlb(-5)
+        m.y.setub(-4)
+        with self.assertRaises(InfeasibleConstraintException):
+            self.tightener(m)
+
+        m.x.setlb(None)
+        m.x.setub(None)
+        m.y.setlb(-5)
+        m.y.setub(0)
+        self.tightener(m)
+        self.assertAlmostEqual(m.x.lb, 0)
+        self.assertAlmostEqual(m.x.ub, 0)
+        self.assertAlmostEqual(m.y.lb, 0)
+        self.assertAlmostEqual(m.y.ub, 0)
+
+        m.x.setlb(-5)
+        m.x.setub(5)
+        m.y.setlb(None)
+        m.y.setub(None)
+        self.tightener(m)
+        self.assertAlmostEqual(m.x.lb, -5)
+        self.assertAlmostEqual(m.x.ub, 5)
+        self.assertAlmostEqual(m.y.lb, 0)
+        self.assertAlmostEqual(m.y.ub, 5)
+
+        m.x.setlb(-5)
+        m.x.setub(-2)
+        m.y.setlb(None)
+        m.y.setub(None)
+        self.tightener(m)
+        self.assertAlmostEqual(m.x.lb, -5)
+        self.assertAlmostEqual(m.x.ub, -2)
+        self.assertAlmostEqual(m.y.lb, 2)
+        self.assertAlmostEqual(m.y.ub, 5)
+
+        m.x.setlb(2)
+        m.x.setub(5)
+        m.y.setlb(None)
+        m.y.setub(None)
+        self.tightener(m)
+        self.assertAlmostEqual(m.x.lb, 2)
+        self.assertAlmostEqual(m.x.ub, 5)
+        self.assertAlmostEqual(m.y.lb, 2)
+        self.assertAlmostEqual(m.y.ub, 5)
+
+
     def test_log(self):
+        if not numpy_available:
+            raise unittest.SkipTest('Numpy is not available')
         c_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         for cl, cu in c_bounds:
             m = pyo.Block(concrete=True)
             m.x = pyo.Var()
             m.c = pyo.Constraint(expr=pyo.inequality(body=pyo.log(m.x), lower=cl, upper=cu))
-            fbbt(m)
+            self.tightener(m)
             z = np.linspace(pyo.value(m.c.lower), pyo.value(m.c.upper), 100)
             if m.x.lb is None:
                 xl = -np.inf
@@ -541,14 +643,15 @@ class TestFBBT(unittest.TestCase):
             self.assertTrue(np.all(xl <= x))
             self.assertTrue(np.all(xu >= x))
 
-    @unittest.skipIf(not numpy_available, 'Numpy is not available.')
     def test_log10(self):
+        if not numpy_available:
+            raise unittest.SkipTest('Numpy is not available')
         c_bounds = [(-2.5, 2.8), (-2.5, -0.5), (0.5, 2.8), (-2.5, 0), (0, 2.8), (-2.5, -1), (1, 2.8), (-1, -0.5), (0.5, 1)]
         for cl, cu in c_bounds:
             m = pyo.Block(concrete=True)
             m.x = pyo.Var()
             m.c = pyo.Constraint(expr=pyo.inequality(body=pyo.log10(m.x), lower=cl, upper=cu))
-            fbbt(m)
+            self.tightener(m)
             z = np.linspace(pyo.value(m.c.lower), pyo.value(m.c.upper), 100)
             if m.x.lb is None:
                 xl = -np.inf
@@ -568,14 +671,14 @@ class TestFBBT(unittest.TestCase):
         m = pyo.Block(concrete=True)
         m.x = pyo.Var(bounds=(-math.pi/2, math.pi/2))
         m.c = pyo.Constraint(expr=pyo.inequality(body=pyo.sin(m.x), lower=-0.5, upper=0.5))
-        fbbt(m.c)
+        self.tightener(m)
         self.assertAlmostEqual(pyo.value(m.x.lb), math.asin(-0.5))
         self.assertAlmostEqual(pyo.value(m.x.ub), math.asin(0.5))
 
         m = pyo.Block(concrete=True)
         m.x = pyo.Var()
         m.c = pyo.Constraint(expr=pyo.inequality(body=pyo.sin(m.x), lower=-0.5, upper=0.5))
-        fbbt(m.c)
+        self.tightener(m)
         self.assertEqual(m.x.lb, None)
         self.assertEqual(m.x.ub, None)
 
@@ -583,14 +686,14 @@ class TestFBBT(unittest.TestCase):
         m = pyo.Block(concrete=True)
         m.x = pyo.Var(bounds=(0, math.pi))
         m.c = pyo.Constraint(expr=pyo.inequality(body=pyo.cos(m.x), lower=-0.5, upper=0.5))
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(pyo.value(m.x.lb), math.acos(0.5))
         self.assertAlmostEqual(pyo.value(m.x.ub), math.acos(-0.5))
 
         m = pyo.Block(concrete=True)
         m.x = pyo.Var()
         m.c = pyo.Constraint(expr=pyo.inequality(body=pyo.cos(m.x), lower=-0.5, upper=0.5))
-        fbbt(m)
+        self.tightener(m)
         self.assertEqual(m.x.lb, None)
         self.assertEqual(m.x.ub, None)
 
@@ -598,14 +701,14 @@ class TestFBBT(unittest.TestCase):
         m = pyo.Block(concrete=True)
         m.x = pyo.Var(bounds=(-math.pi/2, math.pi/2))
         m.c = pyo.Constraint(expr=pyo.inequality(body=pyo.tan(m.x), lower=-0.5, upper=0.5))
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(pyo.value(m.x.lb), math.atan(-0.5))
         self.assertAlmostEqual(pyo.value(m.x.ub), math.atan(0.5))
 
         m = pyo.Block(concrete=True)
         m.x = pyo.Var()
         m.c = pyo.Constraint(expr=pyo.inequality(body=pyo.tan(m.x), lower=-0.5, upper=0.5))
-        fbbt(m)
+        self.tightener(m)
         self.assertEqual(m.x.lb, None)
         self.assertEqual(m.x.ub, None)
 
@@ -613,7 +716,7 @@ class TestFBBT(unittest.TestCase):
         m = pyo.Block(concrete=True)
         m.x = pyo.Var()
         m.c = pyo.Constraint(expr=pyo.inequality(body=pyo.asin(m.x), lower=-0.5, upper=0.5))
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(pyo.value(m.x.lb), math.sin(-0.5))
         self.assertAlmostEqual(pyo.value(m.x.ub), math.sin(0.5))
 
@@ -621,7 +724,7 @@ class TestFBBT(unittest.TestCase):
         m = pyo.Block(concrete=True)
         m.x = pyo.Var()
         m.c = pyo.Constraint(expr=pyo.inequality(body=pyo.acos(m.x), lower=1, upper=2))
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(pyo.value(m.x.lb), math.cos(2))
         self.assertAlmostEqual(pyo.value(m.x.ub), math.cos(1))
 
@@ -629,7 +732,7 @@ class TestFBBT(unittest.TestCase):
         m = pyo.Block(concrete=True)
         m.x = pyo.Var()
         m.c = pyo.Constraint(expr=pyo.inequality(body=pyo.atan(m.x), lower=-0.5, upper=0.5))
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(pyo.value(m.x.lb), math.tan(-0.5))
         self.assertAlmostEqual(pyo.value(m.x.ub), math.tan(0.5))
 
@@ -644,7 +747,7 @@ class TestFBBT(unittest.TestCase):
         m.c.add(m.y - m.x*m.z <= 2)
         m.c.add(m.y - m.x*m.z >= -2)
         m.c.add(m.x + m.z == 1)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(pyo.value(m.x.lb), -1, 8)
         self.assertAlmostEqual(pyo.value(m.x.ub), -1, 8)
         self.assertAlmostEqual(pyo.value(m.y.lb), 0, 8)
@@ -663,7 +766,7 @@ class TestFBBT(unittest.TestCase):
         m.c.add(-m.y - m.x*m.z >= -2)
         m.c.add(-m.y - m.x*m.z <= 2)
         m.c.add(-m.x - m.z == 1)
-        fbbt(m)
+        self.tightener(m)
         self.assertAlmostEqual(pyo.value(m.x.lb), 1, 8)
         self.assertAlmostEqual(pyo.value(m.x.ub), 1, 8)
         self.assertAlmostEqual(pyo.value(m.y.lb), 0, 8)
@@ -676,7 +779,7 @@ class TestFBBT(unittest.TestCase):
         m.x = pyo.Var(domain=pyo.Binary)
         m.y = pyo.Var(domain=pyo.Binary)
         m.c = pyo.Constraint(expr=m.x + m.y >= 1.5)
-        fbbt(m)
+        self.tightener(m)
         self.assertEqual(pyo.value(m.x.lb), 1)
         self.assertEqual(pyo.value(m.x.ub), 1)
         self.assertEqual(pyo.value(m.y.lb), 1)
@@ -686,7 +789,7 @@ class TestFBBT(unittest.TestCase):
         m.x = pyo.Var(domain=pyo.Binary)
         m.y = pyo.Var(domain=pyo.Binary)
         m.c = pyo.Constraint(expr=m.x + m.y <= 0.5)
-        fbbt(m)
+        self.tightener(m)
         self.assertEqual(pyo.value(m.x.lb), 0)
         self.assertEqual(pyo.value(m.x.ub), 0)
         self.assertEqual(pyo.value(m.y.lb), 0)
@@ -697,9 +800,14 @@ class TestFBBT(unittest.TestCase):
         m.x = pyo.Var(bounds=(1,2))
         m.y = pyo.Var(bounds=(1,2))
         m.c = pyo.Constraint(expr=m.x + m.y >= 0)
-        fbbt(m)
+        self.tightener(m)
         self.assertTrue(m.c.active)
-        fbbt(m, deactivate_satisfied_constraints=True)
+        if self.tightener is fbbt:
+            self.tightener(m, deactivate_satisfied_constraints=True)
+        else:
+            self.it.config.deactivate_satisfied_constraints = True
+            self.tightener(m)
+            self.it.config.deactivate_satisfied_constraints = False
         self.assertFalse(m.c.active)
 
     def test_iteration_limit(self):
@@ -712,7 +820,12 @@ class TestFBBT(unittest.TestCase):
         m.c[1] = m.x[1] == m.x[2]
         m.x[2].setlb(-1)
         m.x[2].setub(1)
-        fbbt(m, max_iter=1)
+        if self.tightener is fbbt:
+            self.tightener(m, max_iter=1)
+        else:
+            self.it.config.max_iter = 1
+            self.tightener(m)
+            self.it.config.max_iter = 10
         self.assertEqual(m.x[1].lb, -1)
         self.assertEqual(m.x[1].ub, 1)
         self.assertEqual(m.x[0].lb, None)
@@ -722,11 +835,15 @@ class TestFBBT(unittest.TestCase):
         m = pyo.ConcreteModel()
         m.x = pyo.Var(bounds=(-1, 1))
         m.y = pyo.Var()
-        lb, ub = compute_bounds_on_expr(m.x + m.y)
-        self.assertEqual(lb, None)
-        self.assertEqual(ub, None)
+        m.z = pyo.Var()
+        m.c = pyo.Constraint(expr=m.x + m.y == m.z)
+        self.tightener(m)
+        self.assertEqual(m.z.lb, None)
+        self.assertEqual(m.z.ub, None)
 
     def test_skip_unknown_expression1(self):
+        if self.tightener is not fbbt:
+            raise unittest.SkipTest('Appsi FBBT does not support unkown expressions yet')
 
         m = pyo.ConcreteModel()
         m.x = pyo.Var(bounds=(1,1))
@@ -736,7 +853,7 @@ class TestFBBT(unittest.TestCase):
 
         OUT = StringIO()
         with LoggingIntercept(OUT, 'pyomo.contrib.fbbt.fbbt'):
-            new_bounds = fbbt(m)
+            new_bounds = self.tightener(m)
 
         self.assertEqual(pyo.value(m.x.lb), 1)
         self.assertEqual(pyo.value(m.x.ub), 1)
@@ -745,6 +862,9 @@ class TestFBBT(unittest.TestCase):
         self.assertIn("Unsupported expression type for FBBT", OUT.getvalue())
 
     def test_skip_unknown_expression2(self):
+        if self.tightener is not fbbt:
+            raise unittest.SkipTest('Appsi FBBT does not support unkown expressions yet')
+
         def dummy_unary_expr(x):
             return 0.5*x
 
@@ -755,7 +875,7 @@ class TestFBBT(unittest.TestCase):
 
         OUT = StringIO()
         with LoggingIntercept(OUT, 'pyomo.contrib.fbbt.fbbt'):
-            new_bounds = fbbt(m)
+            new_bounds = self.tightener(m)
 
         self.assertEqual(pyo.value(m.x.lb), 0)
         self.assertEqual(pyo.value(m.x.ub), 4)
@@ -775,7 +895,7 @@ class TestFBBT(unittest.TestCase):
         m.x = pyo.Var(bounds=(-0.035, -0.035))
         m.y = pyo.Var(bounds=(-0.023, -0.023))
         m.c = pyo.Constraint(expr=m.x**2 + m.y**2 <= 0.0256)
-        fbbt(m.c)
+        self.tightener(m)
         self.assertEqual(m.x.lb, -0.035)
         self.assertEqual(m.x.ub, -0.035)
         self.assertEqual(m.y.lb, -0.023)
@@ -786,7 +906,7 @@ class TestFBBT(unittest.TestCase):
         m.x = pyo.Var(within=pyo.Integers)
         m.y = pyo.Var(within=pyo.Integers)
         m.c = pyo.Constraint(expr=m.x + m.y == 1)
-        fbbt(m.c)
+        self.tightener(m)
         self.assertEqual(m.x.lb, None)
         self.assertEqual(m.x.ub, None)
         self.assertEqual(m.y.lb, None)
@@ -804,7 +924,7 @@ class TestFBBT(unittest.TestCase):
 
         m.c = pyo.Constraint(expr=m.x == pyo.sin(m.y))
 
-        fbbt(m.c)
+        self.tightener(m)
 
         self.assertAlmostEqual(m.x.lb, xl)
         self.assertAlmostEqual(m.x.ub, xu)
@@ -815,7 +935,117 @@ class TestFBBT(unittest.TestCase):
         m = pyo.ConcreteModel()
         m.x = pyo.Var()
         m.y = pyo.Var()
-        e = (m.x**2 + m.y**2)**(-0.5)
-        lb, ub = compute_bounds_on_expr(e)
-        self.assertAlmostEqual(lb, 0)
-        self.assertIsNone(ub)
+        m.z = pyo.Var()
+        m.c = pyo.Constraint(expr=(m.x**2 + m.y**2)**(-0.5) == m.z)
+        self.tightener(m)
+        self.assertAlmostEqual(m.z.lb, 0)
+        self.assertIsNone(m.z.ub)
+
+    def test_linear_expression(self):
+        m = pyo.ConcreteModel()
+        m.x = pyo.Var(bounds=(1, 2))
+        m.y = pyo.Var()
+        m.p = pyo.Param(initialize=3, mutable=True)
+        e = LinearExpression(constant=1, linear_coefs=[1, m.p - 1], linear_vars=[m.x, m.y])
+        m.c = pyo.Constraint(expr=e == 0)
+        self.tightener(m)
+        self.assertAlmostEqual(m.y.lb, -1.5)
+        self.assertAlmostEqual(m.y.ub, -1)
+
+    def test_quadratic_as_product(self):
+        m1 = pyo.ConcreteModel()
+        m1.x = pyo.Var([1, 2], bounds=(-2, 6))
+        m1.y = pyo.Var()
+        m1.c = pyo.Constraint(expr=m1.x[1]*m1.x[1] + m1.x[2]*m1.x[2] == m1.y)
+
+        m2 = pyo.ConcreteModel()
+        m2.x = pyo.Var([1, 2], bounds=(-2, 6))
+        m2.y = pyo.Var()
+        m2.c = pyo.Constraint(expr=m2.x[1]**2 + m2.x[2]**2 == m2.y)
+
+        self.tightener(m1)
+        self.tightener(m2)
+
+        self.assertAlmostEqual(m1.y.lb, m2.y.lb)
+        self.assertAlmostEqual(m1.y.ub, m2.y.ub)
+
+        m = pyo.ConcreteModel()
+        m.x = pyo.Var([1, 2], bounds=(-2, 6))
+        m.y = pyo.Var()
+        m.c = pyo.Constraint(expr=m.x[1]*m.x[1] + m.x[2]*m.x[2] == 0)
+        self.tightener(m)
+        self.assertAlmostEqual(m.x[1].lb, 0)
+        self.assertAlmostEqual(m.x[1].ub, 0)
+        self.assertAlmostEqual(m.x[2].lb, 0)
+        self.assertAlmostEqual(m.x[2].ub, 0)
+
+    def test_long_sum(self):
+        N = 30
+        for n in range(N):
+            m = pyo.ConcreteModel()
+            m.a = pyo.Set(initialize=list(range(N)))
+            m.x = pyo.Var(m.a, bounds=(0, 1))
+            m.x[n].setub(None)
+            m.c = pyo.Constraint(expr=sum(m.x.values()) == 1)
+            self.tightener(m)
+            self.assertAlmostEqual(m.x[n].ub, 1)
+
+            m = pyo.ConcreteModel()
+            m.a = pyo.Set(initialize=list(range(N)))
+            m.x = pyo.Var(m.a, bounds=(0, 1))
+            m.x[n].setlb(None)
+            m.c = pyo.Constraint(expr=sum(m.x.values()) == 1)
+            self.tightener(m)
+            self.assertAlmostEqual(m.x[n].lb, -28)
+
+    def test_long_linear_expression(self):
+        N = 30
+        for n in range(N):
+            m = pyo.ConcreteModel()
+            m.a = pyo.Set(initialize=list(range(N)))
+            m.x = pyo.Var(m.a, bounds=(0, 1))
+            m.x[n].setub(None)
+            m.c = pyo.Constraint(expr=LinearExpression(constant=0, linear_coefs=[1]*N, linear_vars=list(m.x.values())) == 1)
+            self.tightener(m)
+            self.assertAlmostEqual(m.x[n].ub, 1)
+
+            m = pyo.ConcreteModel()
+            m.a = pyo.Set(initialize=list(range(N)))
+            m.x = pyo.Var(m.a, bounds=(0, 1))
+            m.x[n].setlb(None)
+            m.c = pyo.Constraint(expr=LinearExpression(constant=0, linear_coefs=[1]*N, linear_vars=list(m.x.values())) == 1)
+            self.tightener(m)
+            self.assertAlmostEqual(m.x[n].lb, -28)
+
+    def test_long_linear_expression2(self):
+        N = 30
+        for n in range(N):
+            m = pyo.ConcreteModel()
+            m.a = pyo.Set(initialize=list(range(N)))
+            m.x = pyo.Var(m.a, bounds=(0, 1))
+            m.x[n].setlb(None)
+            m.x[n].setub(None)
+            m.c = pyo.Constraint(expr=LinearExpression(constant=1, linear_coefs=[1]*N, linear_vars=list(m.x.values())) == 1)
+            self.tightener(m)
+            self.assertAlmostEqual(m.x[n].lb, -29)
+            self.assertAlmostEqual(m.x[n].ub, 0)
+
+    def test_fixed_var(self):
+        m = pyo.ConcreteModel()
+        m.x = pyo.Var()
+        m.y = pyo.Var()
+        m.z = pyo.Var(bounds=(1, 2))
+        m.x.fix(1)
+        m.c = pyo.Constraint(expr=m.z == m.x * m.y)
+
+        self.tightener(m)
+
+        self.assertAlmostEqual(m.x.lb, 1)
+        self.assertAlmostEqual(m.x.ub, 1)
+        self.assertAlmostEqual(m.y.lb, 1)
+        self.assertAlmostEqual(m.y.ub, 2)
+
+
+class TestFBBT(FbbtTestBase, unittest.TestCase):
+    def setUp(self) -> None:
+        self.tightener = fbbt
