@@ -1,7 +1,8 @@
 #  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Copyright (c) 2008-2022
+#  National Technology and Engineering Solutions of Sandia, LLC
 #  Under the terms of Contract DE-NA0003525 with National Technology and
 #  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain
 #  rights in this software.
@@ -16,16 +17,14 @@ from pyomo.common.dependencies import (
     scipy, scipy_available,
     check_min_version, attempt_import
 )
-
-plt = matplotlib.pyplot
-stats = scipy.stats
+from pyomo.common.dependencies.matplotlib import pyplot as plt
+from pyomo.common.dependencies.scipy import stats
 
 # occasionally dependent conda packages for older distributions
 # (e.g. python 3.5) get released that are either broken not
 # compatible, resulting in a SyntaxError
 sns, seaborn_available = attempt_import(
-    'seaborn', alt_names=['sns'],
-    catch_exceptions=(ImportError, SyntaxError)
+    'seaborn', catch_exceptions=(ImportError, SyntaxError)
 )
 
 imports_available = numpy_available & scipy_available & pandas_available \
