@@ -1,7 +1,8 @@
 #  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Copyright (c) 2008-2022
+#  National Technology and Engineering Solutions of Sandia, LLC
 #  Under the terms of Contract DE-NA0003525 with National Technology and
 #  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain
 #  rights in this software.
@@ -9,19 +10,22 @@
 #  ___________________________________________________________________________
 
 import logging
-import six
 import sys
 from copy import deepcopy
 from pickle import PickleError
-from six import iteritems, string_types
 from weakref import ref as weakref_ref
 
-from pyutilib.misc.indent_io import StreamIndenter
-
 import pyomo.common
-from pyomo.common.deprecation import deprecated, relocated_module_attribute
+from pyomo.common import DeveloperError
+from pyomo.common.deprecation import (
+    deprecated, deprecation_warning, relocated_module_attribute)
+from pyomo.common.factory import Factory
+from pyomo.common.formatting import tabular_writer, StreamIndenter
+from pyomo.common.modeling import NOTSET
+from pyomo.common.sorting import sorted_robust
 from pyomo.core.pyomoobject import PyomoObject
-from pyomo.core.base.misc import tabular_writer, sorted_robust
+from pyomo.core.base.component_namer import name_repr, index_repr
+from pyomo.core.base.global_set import UnindexedComponent_index
 
 logger = logging.getLogger('pyomo.core')
 
@@ -29,44 +33,34 @@ relocated_module_attribute(
     'ComponentUID', 'pyomo.core.base.componentuid.ComponentUID',
     version='5.7.2')
 
-def _name_index_generator(idx):
-    """
-    Return a string representation of an index.
-    """
-    def _escape(val):
-        if type(val) is tuple:
-            ans = "(" + ','.join(_escape(_) for _ in val) + ")"
-        else:
-            # We need to quote set members (because people put things
-            # like spaces - or worse commas - in their set names).  Our
-            # plan is to put the strings in single quotes... but that
-            # requires escaping any single quotes in the string... which
-            # in turn requires escaping the escape character.
-            ans = "%s" % (val,)
-            if isinstance(val, six.string_types):
-                ans = ans.replace("\\", "\\\\").replace("'", "\\'")
-                if ',' in ans or "'" in ans:
-                    ans = "'"+ans+"'"
-        return ans
-    if idx.__class__ is tuple:
-        return "[" + ",".join(_escape(i) for i in idx) + "]"
-    else:
-        return "[" + _escape(idx) + "]"
+_ref_types = {type(None), weakref_ref}
+
+class ModelComponentFactoryClass(Factory):
+
+    def register(self, doc=None):
+        def fn(cls):
+            return super(ModelComponentFactoryClass, self).register(
+                cls.__name__, doc)(cls)
+        return fn
+
+ModelComponentFactory = ModelComponentFactoryClass('model component')
 
 
-def name(component, index=None, fully_qualified=False, relative_to=None):
+def name(component, index=NOTSET, fully_qualified=False, relative_to=None):
     """
     Return a string representation of component for a specific
     index value.
     """
-    base = component.getname(fully_qualified=fully_qualified, relative_to=relative_to)
-    if index is None:
+    base = component.getname(
+        fully_qualified=fully_qualified, relative_to=relative_to
+    )
+    if index is NOTSET:
         return base
     else:
         if index not in component.index_set():
             raise KeyError( "Index %s is not valid for component %s"
                             % (index, component.name) )
-        return base + _name_index_generator( index )
+        return base + index_repr( index )
 
 
 @deprecated(msg="The cname() function has been renamed to name()",
@@ -201,7 +195,7 @@ class _ComponentBase(PyomoObject):
             elif paranoid is not None:
                 raise PickleError()
             new_state = {}
-            for k,v in iteritems(state):
+            for k, v in state.items():
                 try:
                     if paranoid:
                         saved_memo = dict(memo)
@@ -268,7 +262,7 @@ class _ComponentBase(PyomoObject):
         """
         comp = self.parent_component()
         _attr, _data, _header, _fcn = comp._pprint()
-        if isinstance(type(_data), six.string_types):
+        if isinstance(type(_data), str):
             # If the component _pprint only returned a pre-formatted
             # result, then we have no way to only emit the information
             # for this _data object.
@@ -333,6 +327,10 @@ class _ComponentBase(PyomoObject):
             # The first line should be a hanging indent (i.e., not indented)
             ostream.newline = False
 
+        if self.is_reference():
+            _attr = list(_attr) if _attr else []
+            _attr.append(('ReferenceTo', self.referent))
+
         if _name:
             ostream.write(_name+" : ")
         if _doc:
@@ -359,7 +357,7 @@ class _ComponentBase(PyomoObject):
         if _header is not None:
             if _fcn2 is not None:
                 _data_dict = dict(_data)
-                _data = iteritems(_data_dict)
+                _data = _data_dict.items()
             tabular_writer( ostream, '', _data, _header, _fcn )
             if _fcn2 is not None:
                 for _key in sorted_robust(_data_dict):
@@ -406,7 +404,7 @@ class Component(_ComponentBase):
         # Verify that ctype has been specified.
         #
         if self._ctype is None:
-            raise pyomo.common.DeveloperError(
+            raise DeveloperError(
                 "Must specify a component type for class %s!"
                 % ( type(self).__name__, ) )
         #
@@ -433,7 +431,7 @@ class Component(_ComponentBase):
         _base = super(Component,self)
         if hasattr(_base, '__getstate__'):
             state = _base.__getstate__()
-            for key,val in iteritems(self.__dict__):
+            for key,val in self.__dict__.items():
                 if key not in state:
                     state[key] = val
         else:
@@ -447,8 +445,7 @@ class Component(_ComponentBase):
         This method must be defined to support pickling because this class
         owns weakrefs for '_parent'.
         """
-        if state['_parent'] is not None and \
-                type(state['_parent']) is not weakref_ref:
+        if state['_parent'].__class__ not in _ref_types:
             state['_parent'] = weakref_ref(state['_parent'])
         #
         # Note: our model for setstate is for derived classes to modify
@@ -461,7 +458,7 @@ class Component(_ComponentBase):
         if hasattr(_base, '__setstate__'):
             _base.__setstate__(state)
         else:
-            for key, val in iteritems(state):
+            for key, val in state.items():
                 # Note: per the Python data model docs, we explicitly
                 # set the attribute using object.__setattr__() instead
                 # of setting self.__dict__[key] = val.
@@ -487,9 +484,23 @@ class Component(_ComponentBase):
         return self._constructed
 
     def reconstruct(self, data=None):
-        """Re-construct model expressions"""
-        self._constructed = False
-        self.construct(data=data)
+        """REMOVED: reconstruct() was removed in Pyomo 6.0.
+
+        Re-constructing model components was fragile and did not
+        correctly update instances of the component used in other
+        components or contexts (this was particularly problemmatic for
+        Var, Param, and Set).  Users who wish to reproduce the old
+        behavior of reconstruct(), are comfortable manipulating
+        non-public interfaces, and who take the time to verify that the
+        correct thing happens to their model can approximate the old
+        behavior of reconstruct with:
+
+            component.clear()
+            component._constructed = False
+            component.construct()
+
+        """
+        raise AttributeError(self.reconstruct.__doc__)
 
     def valid_model_component(self):
         """Return True if this can be used as a model component."""
@@ -541,15 +552,6 @@ class Component(_ComponentBase):
         """Return the component name"""
         return self.name
 
-    def to_string(self, verbose=None, labeler=None, smap=None, compute_values=False):
-        """Return the component name"""
-        if compute_values:
-            try:
-                return str(self())
-            except:
-                pass
-        return self.name
-
     def getname(self, fully_qualified=False, name_buffer=None, relative_to=None):
         """Returns the component name associated with this object.
 
@@ -558,30 +560,36 @@ class Component(_ComponentBase):
         fully_qualified: bool
             Generate full name from nested block names
 
-        name_buffer: dict
-            A dictionary that caches encountered names and indices.
-            Providing a ``name_buffer`` can significantly speed up
-            iterative name generation
-
         relative_to: Block
             Generate fully_qualified names reletive to the specified block.
         """
+        local_name = self._name
         if fully_qualified:
             pb = self.parent_block()
             if relative_to is None:
                 relative_to = self.model()
             if pb is not None and pb is not relative_to:
                 ans = pb.getname(fully_qualified, name_buffer, relative_to) \
-                      + "." + self._name
+                      + "." + name_repr(local_name)
             elif pb is None and relative_to != self.model():
                 raise RuntimeError(
                     "The relative_to argument was specified but not found "
                     "in the block hierarchy: %s" % str(relative_to))
             else:
-                ans = self._name
+                ans = name_repr(local_name)
         else:
-            ans = self._name
+            # Note: we want "getattr(x.parent_block(), x.local_name) == x"
+            # so we do not want to call _safe_name_str, as that could
+            # add quotes or otherwise escape the string.
+            ans = local_name
         if name_buffer is not None:
+            deprecation_warning(
+                "The 'name_buffer' argument to getname is deprecated. "
+                "The functionality is no longer necessary since getting names "
+                "is no longer a quadratic operation. Additionally, note that "
+                "use of this argument poses risks if the buffer contains "
+                "names relative to different Blocks in the model hierarchy or "
+                "a mixture of local and fully_qualified names.", version='TODO')
             name_buffer[id(self)] = ans
         return ans
 
@@ -608,7 +616,7 @@ class Component(_ComponentBase):
 
     def clear_suffix_value(self, suffix_or_name, expand=True):
         """Clear the suffix value for this component data"""
-        if isinstance(suffix_or_name, six.string_types):
+        if isinstance(suffix_or_name, str):
             import pyomo.core.base.suffix
             for name_, suffix_ in pyomo.core.base.suffix.active_suffix_generator(self.model()):
                 if suffix_or_name == name_:
@@ -619,7 +627,7 @@ class Component(_ComponentBase):
 
     def set_suffix_value(self, suffix_or_name, value, expand=True):
         """Set the suffix value for this component data"""
-        if isinstance(suffix_or_name, six.string_types):
+        if isinstance(suffix_or_name, str):
             import pyomo.core.base.suffix
             for name_, suffix_ in pyomo.core.base.suffix.active_suffix_generator(self.model()):
                 if suffix_or_name == name_:
@@ -630,7 +638,7 @@ class Component(_ComponentBase):
 
     def get_suffix_value(self, suffix_or_name, default=None):
         """Get the suffix value for this component data"""
-        if isinstance(suffix_or_name, six.string_types):
+        if isinstance(suffix_or_name, str):
             import pyomo.core.base.suffix
             for name_, suffix_ in pyomo.core.base.suffix.active_suffix_generator(self.model()):
                 if suffix_or_name == name_:
@@ -688,11 +696,20 @@ class ComponentData(_ComponentBase):
 
     Private class attributes:
         _component      A weakref to the component that owns this data object
+        _index          The index of this data object
         """
 
-    __pickle_slots__ = ('_component',)
+    __pickle_slots__ = ('_component', '_index')
     __slots__ = __pickle_slots__ + ('__weakref__',)
 
+    # NOTE: This constructor is in-lined in the constructors for the following
+    # classes: _BooleanVarData, _ConnectorData, _ConstraintData,
+    # _GeneralExpressionData, _LogicalConstraintData,
+    # _GeneralLogicalConstraintData, _GeneralObjectiveData,
+    # _ParamData,_GeneralVarData, _GeneralBooleanVarData, _DisjunctionData,
+    # _ArcData, _PortData, _LinearConstraintData, and
+    # _LinearMatrixConstraintData. Changes made here need to be made in those
+    # constructors as well!
     def __init__(self, component):
         #
         # ComponentData objects are typically *private* objects for
@@ -702,6 +719,7 @@ class ComponentData(_ComponentBase):
         # this assumption is significantly faster.
         #
         self._component = weakref_ref(component)
+        self._index = NOTSET
 
     def __getstate__(self):
         """Prepare a picklable state of this instance for pickling.
@@ -739,6 +757,7 @@ class ComponentData(_ComponentBase):
             state['_component'] = None
         else:
             state['_component'] = self._component()
+        state['_index'] = self._index
         return state
 
     def __setstate__(self, state):
@@ -759,8 +778,7 @@ class ComponentData(_ComponentBase):
         # we don't the model cloning appears to fail (in the Benders
         # example)
         #
-        if state['_component'] is not None and \
-                type(state['_component']) is not weakref_ref:
+        if state['_component'].__class__ not in _ref_types:
             state['_component'] = weakref_ref(state['_component'])
         #
         # Note: our model for setstate is for derived classes to modify
@@ -773,7 +791,7 @@ class ComponentData(_ComponentBase):
         if hasattr(_base, '__setstate__'):
             _base.__setstate__(state)
         else:
-            for key, val in iteritems(state):
+            for key, val in state.items():
                 # Note: per the Python data model docs, we explicitly
                 # set the attribute using object.__setattr__() instead
                 # of setting self.__dict__[key] = val.
@@ -831,47 +849,48 @@ class ComponentData(_ComponentBase):
         to the parent component index set. None is returned if
         this instance does not have a parent component, or if
         - for some unknown reason - this instance does not belong
-        to the parent component's index set. This method is not
-        intended to be a fast method;  it should be used rarely,
-        primarily in cases of label formulation.
+        to the parent component's index set.
         """
-        self_component = self.parent_component()
-        if self_component is None:
-            return None
-        for idx, component_data in self_component.iteritems():
-            if component_data is self:
-                return idx
-        return None
+        parent = self.parent_component()
+        if ( parent is not None and
+             self._index is not NOTSET and
+             parent[self._index] is not self ):
+            # This error message is a bit goofy, but we can't call self.name
+            # here--it's an infinite loop!
+            raise DeveloperError(
+                "The '_data' dictionary and '_index' attribute are out of "
+                "sync for indexed %s '%s': The %s entry in the '_data' "
+                "dictionary does not map back to this component data object."
+                % (parent.ctype.__name__, parent.name, self._index))
+        return self._index
 
     def __str__(self):
         """Return a string with the component name and index"""
         return self.name
 
-    def to_string(self, verbose=None, labeler=None, smap=None, compute_values=False):
-        """
-        Return a string representation of this component,
-        applying the labeler if passed one.
-        """
-        if compute_values:
-            try:
-                return str(self())
-            except:
-                pass
-        if smap:
-            return smap.getSymbol(self, labeler)
-        if labeler is not None:
-            return labeler(self)
-        else:
-            return self.__str__()
-
     def getname(self, fully_qualified=False, name_buffer=None, relative_to=None):
         """Return a string with the component name and index"""
+        # NOTE: There are bugs with name buffers if a user always gives the same
+        # dictionary but switches from fully-qualified to local names or changes
+        # the component the name is relative to. We will simply deprecate the
+        # buffer in a future PR, but for now we will leave this method so it
+        # behaves the same when a buffer is given, and, in the absence of a
+        # buffer it will construct the name using the index (woohoo!)
+
         #
         # Using the buffer, which is a dictionary:  id -> string
         #
-        if name_buffer is not None and id(self) in name_buffer:
-            # Return the name if it is in the buffer
-            return name_buffer[id(self)]
+        if name_buffer is not None:
+            deprecation_warning(
+                "The 'name_buffer' argument to getname is deprecated. "
+                "The functionality is no longer necessary since getting names "
+                "is no longer a quadratic operation. Additionally, note that "
+                "use of this argument poses risks if the buffer contains "
+                "names relative to different Blocks in the model hierarchy or "
+                "a mixture of local and fully_qualified names.", version='TODO')
+            if id(self) in name_buffer:
+                # Return the name if it is in the buffer
+                return name_buffer[id(self)]
 
         c = self.parent_component()
         if c is self:
@@ -898,20 +917,17 @@ class ComponentData(_ComponentBase):
         if name_buffer is not None:
             # Iterate through the dictionary and generate all names in
             # the buffer
-            for idx, obj in iteritems(c):
-                name_buffer[id(obj)] = base + _name_index_generator(idx)
+            for idx, obj in c.items():
+                name_buffer[id(obj)] = base + index_repr(idx)
             if id(self) in name_buffer:
                 # Return the name if it is in the buffer
                 return name_buffer[id(self)]
         else:
             #
-            # No buffer, so we iterate through the component _data
-            # dictionary until we find this object.  This can be much
-            # more expensive than if a buffer is provided.
+            # No buffer, we can do what we are going to do all the time after we
+            # deprecate the buffer.
             #
-            for idx, obj in iteritems(c):
-                if obj is self:
-                    return base + _name_index_generator(idx)
+            return base + index_repr(self.index())
         #
         raise RuntimeError("Fatal error: cannot find the component data in "
                            "the owning component's _data dictionary.")
@@ -922,7 +938,7 @@ class ComponentData(_ComponentBase):
 
     def clear_suffix_value(self, suffix_or_name, expand=True):
         """Set the suffix value for this component data"""
-        if isinstance(suffix_or_name, six.string_types):
+        if isinstance(suffix_or_name, str):
             import pyomo.core.base.suffix
             for name_, suffix_ in pyomo.core.base.suffix.active_suffix_generator(self.model()):
                 if suffix_or_name == name_:
@@ -933,7 +949,7 @@ class ComponentData(_ComponentBase):
 
     def set_suffix_value(self, suffix_or_name, value, expand=True):
         """Set the suffix value for this component data"""
-        if isinstance(suffix_or_name, six.string_types):
+        if isinstance(suffix_or_name, str):
             import pyomo.core.base.suffix
             for name_, suffix_ in pyomo.core.base.suffix.active_suffix_generator(self.model()):
                 if suffix_or_name == name_:
@@ -944,7 +960,7 @@ class ComponentData(_ComponentBase):
 
     def get_suffix_value(self, suffix_or_name, default=None):
         """Get the suffix value for this component data"""
-        if isinstance(suffix_or_name, six.string_types):
+        if isinstance(suffix_or_name, str):
             import pyomo.core.base.suffix
             for name_, suffix_ in pyomo.core.base.suffix.active_suffix_generator(self.model()):
                 if suffix_or_name == name_:
@@ -972,6 +988,7 @@ class ActiveComponentData(ComponentData):
 
     Private class attributes:
         _component      A weakref to the component that owns this data object
+        _index          The index of this data object
         _active         A boolean that indicates whether this data is active
     """
 
@@ -1011,4 +1028,3 @@ class ActiveComponentData(ComponentData):
     def deactivate(self):
         """Set the active attribute to False"""
         self._active = False
-
