@@ -1,7 +1,8 @@
 #  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Copyright (c) 2008-2022
+#  National Technology and Engineering Solutions of Sandia, LLC
 #  Under the terms of Contract DE-NA0003525 with National Technology and
 #  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain
 #  rights in this software.
@@ -19,15 +20,16 @@ __all__ = ('Objective',
 import sys
 import logging
 from weakref import ref as weakref_ref
+from typing import overload
 
 from pyomo.common.log import is_debug_set
+from pyomo.common.modeling import NOTSET
 from pyomo.common.deprecation import RenamedClass
 from pyomo.common.formatting import tabular_writer
 from pyomo.common.timing import ConstructionTimer
 from pyomo.core.expr.numvalue import value
-from pyomo.core.base.component import (
-    ActiveComponentData, ModelComponentFactory,
-)
+from pyomo.core.base.component import ActiveComponentData, ModelComponentFactory
+from pyomo.core.base.global_set import UnindexedComponent_index
 from pyomo.core.base.indexed_component import (
     ActiveIndexedComponent, UnindexedComponent_set, rule_wrapper,
 )
@@ -149,6 +151,7 @@ class _GeneralObjectiveData(_GeneralExpressionDataImpl,
         # Inlining ActiveComponentData.__init__
         self._component = weakref_ref(component) if (component is not None) \
                           else None
+        self._index = NOTSET
         self._active = True
         self._sense = sense
 
@@ -213,10 +216,10 @@ class Objective(ActiveIndexedComponent):
             A function that is used to construct objective expressions
         sense
             Indicate whether minimizing (the default) or maximizing
-        doc
-            A text string describing this component
         name
             A name for this component
+        doc
+            A text string describing this component
 
     Public class attributes:
         doc
@@ -259,6 +262,10 @@ class Objective(ActiveIndexedComponent):
         else:
             return IndexedObjective.__new__(IndexedObjective)
 
+    @overload
+    def __init__(self, *indexes, expr=None, rule=None, sense=minimize,
+                 name=None, doc=None): ...
+
     def __init__(self, *args, **kwargs):
         _sense = kwargs.pop('sense', minimize)
         _init = tuple( _arg for _arg in (
@@ -359,7 +366,7 @@ class Objective(ActiveIndexedComponent):
         """
         return (
             [("Size", len(self)),
-             ("Index", self._index if self.is_indexed() else None),
+             ("Index", self._index_set if self.is_indexed() else None),
              ("Active", self.active)
              ],
             self._data.items(),
@@ -380,7 +387,7 @@ class Objective(ActiveIndexedComponent):
         ostream.write(prefix+self.local_name+" : ")
         ostream.write(", ".join("%s=%s" % (k,v) for k,v in [
                     ("Size", len(self)),
-                    ("Index", self._index if self.is_indexed() else None),
+                    ("Index", self._index_set if self.is_indexed() else None),
                     ("Active", self.active),
                     ] ))
 
@@ -400,6 +407,7 @@ class ScalarObjective(_GeneralObjectiveData, Objective):
     def __init__(self, *args, **kwd):
         _GeneralObjectiveData.__init__(self, expr=None, component=self)
         Objective.__init__(self, *args, **kwd)
+        self._index = UnindexedComponent_index
 
     #
     # Since this class derives from Component and
@@ -546,6 +554,7 @@ class ObjectiveList(IndexedObjective):
             raise ValueError(
                 "ObjectiveList does not accept the 'expr' keyword")
         _rule = kwargs.pop('rule', None)
+        self._starting_index = kwargs.pop('starting_index', 1)
 
         args = (Set(dimen=1),)
         super().__init__(*args, **kwargs)
@@ -555,7 +564,9 @@ class ObjectiveList(IndexedObjective):
         # after the base class is set up so that is_indexed() is
         # reliable.
         if self.rule is not None and type(self.rule) is IndexedCallInitializer:
-            self.rule = CountedCallInitializer(self, self.rule)
+            self.rule = CountedCallInitializer(
+                self, self.rule, self._starting_index
+            )
 
     def construct(self, data=None):
         """
@@ -582,8 +593,8 @@ class ObjectiveList(IndexedObjective):
 
     def add(self, expr, sense=minimize):
         """Add an objective to the list."""
-        next_idx = len(self._index) + 1
-        self._index.add(next_idx)
+        next_idx = len(self._index_set) + self._starting_index
+        self._index_set.add(next_idx)
         ans = self.__setitem__(next_idx, expr)
         if ans is not None:
             if sense not in {minimize, maximize}:
